/* GIMP - The GNU Image Manipulation Program
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __SPLASH_H__
#define __SPLASH_H__

void   splash_create    (Gimp         *gimp,
                         gboolean      be_verbose,
                         GdkMonitor   *monitor,
                         GimpApp      *app);
void   splash_destroy   (void);

void   splash_update    (const gchar *label1,
                         const gchar *label2,
                         gdouble      percentage);


#endif /* __SPLASH_H__ */
