#ifndef __GIT_VERSION_H__
#define __GIT_VERSION_H__

#define GIMP_GIT_VERSION          "GIMP_3_0_2"
#define GIMP_GIT_VERSION_ABBREV   "3d4503e"
#define GIMP_GIT_LAST_COMMIT_YEAR "2025"

#endif /* __GIT_VERSION_H__ */
