/* GIMP - The GNU Image Manipulation Program
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * gimpcompressioncombobox.h
 * Copyright (C) 2019 Ell
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __GIMP_COMPRESSION_COMBO_BOX_H__
#define __GIMP_COMPRESSION_COMBO_BOX_H__


#define GIMP_TYPE_COMPRESSION_COMBO_BOX            (gimp_compression_combo_box_get_type ())
#define GIMP_COMPRESSION_COMBO_BOX(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), GIMP_TYPE_COMPRESSION_COMBO_BOX, GimpCompressionComboBox))
#define GIMP_COMPRESSION_COMBO_BOX_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), GIMP_TYPE_COMPRESSION_COMBO_BOX, GimpCompressionComboBoxClass))
#define GIMP_IS_COMPRESSION_COMBO_BOX(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GIMP_TYPE_COMPRESSION_COMBO_BOX))
#define GIMP_IS_COMPRESSION_COMBO_BOX_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GIMP_TYPE_COMPRESSION_COMBO_BOX))
#define GIMP_COMPRESSION_COMBO_BOX_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), GIMP_TYPE_COMPRESSION_COMBO_BOX, GimpCompressionComboBoxClass))


typedef struct _GimpCompressionComboBoxClass  GimpCompressionComboBoxClass;


struct _GimpCompressionComboBox
{
  GimpStringComboBox  parent_instance;
};

struct _GimpCompressionComboBoxClass
{
  GimpStringComboBoxClass  parent_instance;
};


GType       gimp_compression_combo_box_get_type        (void) G_GNUC_CONST;

GtkWidget * gimp_compression_combo_box_new             (void);

void        gimp_compression_combo_box_set_compression (GimpCompressionComboBox *combo_box,
                                                        const gchar             *compression);
gchar     * gimp_compression_combo_box_get_compression (GimpCompressionComboBox *combo_box);

#endif  /* __GIMP_COMPRESSION_COMBO_BOX_H__ */
