/* GIMP - The GNU Image Manipulation Program
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * gimpoperationdissolve.h
 * Copyright (C) 2012 Ville Sokk <ville.sokk@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __GIMP_OPERATION_DISSOLVE_H__
#define __GIMP_OPERATION_DISSOLVE_H__


#include "gimpoperationlayermode.h"


#define GIMP_TYPE_OPERATION_DISSOLVE            (gimp_operation_dissolve_get_type ())
#define GIMP_OPERATION_DISSOLVE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), GIMP_TYPE_OPERATION_DISSOLVE, GimpOperationDissolve))
#define GIMP_OPERATION_DISSOLVE_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass),  GIMP_TYPE_OPERATION_DISSOLVE, GimpOperationDissolveClass))
#define GIMP_IS_OPERATION_DISSOLVE(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GIMP_TYPE_OPERATION_DISSOLVE))
#define GIMP_IS_OPERATION_DISSOLVE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass),  GIMP_TYPE_OPERATION_DISSOLVE))
#define GIMP_OPERATION_DISSOLVE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj),  GIMP_TYPE_OPERATION_DISSOLVE, GimpOperationDissolveClass))


typedef struct _GimpOperationDissolve      GimpOperationDissolve;
typedef struct _GimpOperationDissolveClass GimpOperationDissolveClass;

struct _GimpOperationDissolveClass
{
  GimpOperationLayerModeClass parent_class;
};

struct _GimpOperationDissolve
{
  GimpOperationLayerMode parent_instance;
};


GType   gimp_operation_dissolve_get_type (void) G_GNUC_CONST;


#endif /* __GIMP_OPERATION_DISSOLVE_H__ */
